function [avgCorrVector, stdCorrVector, avgL1Vector, stdL1Vector] = compare_SRS_MC_Pfold (numRoadmapsPerNumNodes,...
						  numNodeList, ...
						  relativeDirName, ...
						  MC_pfoldResultsFilename,...
						  runIndexOfFirstFile, ...
						  numMCResultsPerNode,...
						  UNINITIALIZED)
%SYNOPSYS

%this finds the average and standard deviation of (correlation
%coefficient, L1 distance)  between the pfold of the roadmaps 
%and the pfold obtained from MC simulations.

%PARAMETERS

%roadmap pfold result files are labeled pfold%d_%d.txt,
%numNodeList(i), runIndexOfFirstFile+k, where k = 0..numRoadmapsPerNumNodes-1.
%numMCResultsPerNode corresponds to how many lines correspond to a
%starting conformation in MC pfold results file. 

%NOTES

%MC pfold results are assumed to be 0-based, and thus the indices 
%are properly adjusted.

% in the MC_pfoldResultsFilename, columns are:
% index #folded #unfolded #MCSimulations

if (numMCResultsPerNode < 1)
  error('numMCResultsPerNode should be >=1\n');
end
        
MCpfoldMatrix = readMCPfoldMatrix(MC_pfoldResultsFilename, ...
				  numMCResultsPerNode);
indices     = MCpfoldMatrix(:,1) + 1;
fprintf(1, 'correlation is computed between total %d data points\n', length(indices));



[SRSpfoldMatrix, numRoadmapsUnusedGivenRoadmapSize] = readSRSPfoldMatrix(numNodeList, numRoadmapsPerNumNodes, runIndexOfFirstFile, relativeDirName);

%[avgCorrVector,varCorrVector] = correlate_SRS_MC_Pfolds  (SRSpfoldMatrix, MCpfoldMatrix);
 
[corrCell, L1Cell] = compare_SRS_MC_Pfolds (SRSpfoldMatrix, ...
					    MCpfoldMatrix, ...
					    numMCResultsPerNode, numNodeList);
%keyboard;

%corrCell
%L1Cell
avgCorrVector = averageCell(corrCell, numMCResultsPerNode, numRoadmapsUnusedGivenRoadmapSize,UNINITIALIZED);
avgL1Vector   = averageCell(L1Cell  , numMCResultsPerNode, ...
			    numRoadmapsUnusedGivenRoadmapSize, UNINITIALIZED);

stdCorrVector = stdOfCellVector  (corrCell, numMCResultsPerNode, numRoadmapsUnusedGivenRoadmapSize,UNINITIALIZED);
stdL1Vector   = stdOfCellVector  (L1Cell, numMCResultsPerNode, numRoadmapsUnusedGivenRoadmapSize,UNINITIALIZED);

%[avgL1Vector,  varL1Vector]   = L1_distance_SRS_MC_Pfolds(SRSpfoldMatrix, MCpfoldMatrix);
plotNumUsedRoadmaps(numNodeList, numRoadmapsPerNumNodes, numRoadmapsUnusedGivenRoadmapSize)

%%%
function scatterPlot(MCindex, MCpfoldVector, SRS_pfoldVector, ...
		     numNodeList, numNodesIndex, roadmapIndex)
figure;
plot(MCpfoldVector, SRS_pfoldVector, '*');
xlabel('MC pfold')
ylabel('SRS pfold');
titleStr = ['comparison with MC of ' num2str((MCindex) * ...
					     100) ' simulations and roadmap' ...
	    ' of ' num2str(numNodeList(numNodesIndex)) ' nodes for' ...
	    ' roadmap ' num2str(roadmapIndex)];
title(titleStr);
hold
plot([0 1], [0 1]);

%%%
function plotNumUsedRoadmaps(numNodeList, numRoadmapsPerNumNodes, numRoadmapsUnusedGivenRoadmapSize)
figure; 
plot(numNodeList, numRoadmapsPerNumNodes-numRoadmapsUnusedGivenRoadmapSize, '*')
ylabel('num roadmaps used in each case');
title('how many roadmaps were utilized to obtain the correlation');
xlabel('roadmap size');

%%%
function MCpfoldMatrix = readMCPfoldMatrix(MC_pfoldResultsFilename, ...
					   numMCResultsPerNode)


MCpfoldData        = load (MC_pfoldResultsFilename);
[numRows,ignore]   = size (MCpfoldData);
fprintf(1, 'will read %d rows of pfold data\n', numRows);

if (mod(numRows, numMCResultsPerNode) ~= 0)
  fprintf(1, 'num rows (in MC pfold results file, %s) mod %d should be 0\n', MC_pfoldResultsFilename, numMCResultsPerNode);
  keyboard;
end

if  (MCpfoldData(1,1) ~= 0)
  fprintf(1, ['warning: the indices in MC simulation result pfolds' ...
	      'should be 0-based. Double check your i/p file.. \n']);
%  keyboard;
end

MCpfoldMatrix      = [];% MCpfoldMatrix is a matrix of numMCResultsPerNode columns, read from MC_pfoldResultsFilename
%this matrix will be of size (indices x numMCResultsPerNode).
%MCpfoldMatrix(i,1) will be the index of the node, and 
%MCpfoldMatrix(i,2..numMCResultsPerNode+1) will be the pfold
%obtained with 100,200,...1000 MC simulations (or 10,20,.. whatever)


rowIndex = 1;          
while (rowIndex <= numRows)
   
  index       = MCpfoldData(rowIndex,1);

  for i = 1:numMCResultsPerNode - 1
    if ((MCpfoldData(rowIndex+i,1) ~= index)) 
      error('MC pfold file has an error or specified numMCResultsPerNode is wrong');
    end
  end

  MCpfolds     = [];
  
  for i = 0:numMCResultsPerNode-1
    MCpfold    = MCpfoldData(rowIndex+i,2) / (MCpfoldData(rowIndex+i,2) + MCpfoldData(rowIndex+i,3));
    MCpfolds   = [MCpfolds MCpfold];
  end
  
  newRow      = [index MCpfolds];
  MCpfoldMatrix = [MCpfoldMatrix; newRow];
  rowIndex    = rowIndex+numMCResultsPerNode;
end

if (length(MCpfoldMatrix(:,1)) ~= numRows/numMCResultsPerNode)
  fprintf(1, 'there is an error here. %d is not equal here to %f\n', length(MCpfoldMatrix(:,1)), numRows/numMCResultsPerNode);
end

function [oneRoadmap_MC_Corr, oneRoadmap_MC_L1] = oneRoadmap_MC_Compare(SRS_pfoldVector, MCpfold)

corrCoefMatrix     =    corrcoef(SRS_pfoldVector,MCpfold);
oneRoadmap_MC_Corr =    corrCoefMatrix(1,2);
oneRoadmap_MC_L1   =    sum(abs(SRS_pfoldVector - MCpfold));


function [roadmap_MC_corr_avg, roadmap_MC_L1_avg] = computeAvgRoadmapMCComparison(oneLessRoadmap, ...
						  numRoadmapsPerNumNodes, ...
						  roadmap_MC_corr_sum, ...
						  roadmap_MC_L1_sum, numDataPts,UNINITIALIZED)

if (oneLessRoadmap ~= numRoadmapsPerNumNodes)
  roadmap_MC_corr_avg    = roadmap_MC_corr_sum    / (numRoadmapsPerNumNodes - oneLessRoadmap);
  roadmap_MC_L1_avg      = roadmap_MC_L1_sum      / ((numRoadmapsPerNumNodes - oneLessRoadmap) * numDataPts);
else
  roadmap_MC_corr_avg     = UNINITIALIZED;
  roadmap_MC_L1_avg       = UNINITIALIZED;
end

%although the function below is a single line, I had to separate it
%otherwise emacs thinks that the for loop does not have a
%corresponding end, and when saving annoys me by saying there is an
%unterminated block, whether I should proceed with save.
function [corrSum, L1Sum] = computeRoadmap_MC_cmp_result_sum(SRS_pfoldVector, ...
						  MCpfold, ...
						  roadmap_MC_corr_sum, roadmap_MC_L1_sum)

[oneRoadmap_MC_Corr, oneRoadmap_MC_L1] = oneRoadmap_MC_Compare(SRS_pfoldVector, MCpfold);
corrSum                                = roadmap_MC_corr_sum + oneRoadmap_MC_Corr;
L1Sum                                  = roadmap_MC_L1_sum   + ...
    oneRoadmap_MC_L1;

function [SRSpfoldMatrix, numRoadmapsUnusedGivenRoadmapSize] = ...
    readSRSPfoldMatrix(numNodeList, numRoadmapsPerNumNodes, ...
		       runIndexOfFirstFile, relativeDirName)

m = length(numNodeList);
n = numRoadmapsPerNumNodes;

SRSpfoldMatrix = cell(m, n);

numRoadmapsUnusedGivenRoadmapSize = zeros(m,1);

for i     = 1:m
  for j     = 1:n
  
    SRS_pfoldFilename       = sprintf('%s/pfold%d_%d.txt', relativeDirName, ...
				      numNodeList(i), j + ...
				      runIndexOfFirstFile - 1);
      
    SRS_pfoldVector         = load   (SRS_pfoldFilename);

    
    if (length(SRS_pfoldVector) == 0)
      numRoadmapsUnusedGivenRoadmapSize(i)  = numRoadmapsUnusedGivenRoadmapSize(i)  + 1;
    else
      SRSpfoldMatrix {i,j} = SRS_pfoldVector;
    end  
  end % for j
end %for i

%%%


function [corrCell, L1Cell] = compare_SRS_MC_Pfolds ...
    (SRSpfoldMatrix, MCpfoldMatrix, numMCResultsPerNode, numNodeList)

if (numMCResultsPerNode ~= length(MCpfoldMatrix(1,:)) - 1)
  error('numMCResultsPerNode ~= length(MCpfoldMatrix(1,:)) - 1\n');
end

[numNodeListSize,numRoadmaps]         = size(SRSpfoldMatrix);

corrCell = cell(numMCResultsPerNode, numNodeListSize, ...
		  numRoadmaps);
L1Cell   = corrCell;

indices       = MCpfoldMatrix(:,1) + 1;

scatterPlotDone = 0;
for MCindex = 1:numMCResultsPerNode

  MC_pfoldVector = MCpfoldMatrix(:, MCindex+1);

  for numNodesIndex = 1:numNodeListSize
    for roadmapIndex = 1:numRoadmaps
      SRSpfoldVector  = SRSpfoldMatrix{numNodesIndex, roadmapIndex};
      
      if (length(SRSpfoldVector) ~= 0)
	[oneCorr, oneL1] = oneRoadmap_MC_Compare(SRSpfoldVector(indices), MC_pfoldVector);
	corrCell{MCindex, numNodesIndex, roadmapIndex} = oneCorr;
	L1Cell  {MCindex, numNodesIndex, roadmapIndex} = oneL1 / length(indices);
	if ((numNodesIndex == numNodeListSize) & (MCindex == ...
						 numMCResultsPerNode) ...
	    & (~scatterPlotDone))
	  scatterPlotDone = 1;
	  scatterPlot(MCindex, MC_pfoldVector, SRSpfoldVector(indices), ...
		      numNodeList, numNodesIndex, roadmapIndex);
	end
      end
    end
  end
end

function avg = averageCell(c, numMCResultsPerNode, numRoadmapsMissing, ...
			   uninitialized)
[m,n,p] = size(c);
%n
%p

if (length(numRoadmapsMissing) ~= n)
  error('length(numRoadmapsMissing) ~= n)\n');
end
%keyboard;
if (m ~= numMCResultsPerNode)
  error ('m should be %d\n', numMCResultsPerNode);
end
%n == numNodeListSize
%p == numRoadmapsPerNumNode
avg = zeros(numMCResultsPerNode, n);

for MCindex = 1:numMCResultsPerNode
  for nodeIndex = 1:n
    sum = 0; numUnusedRoadmap = 0;
    for roadmapIndex = 1:p
      if (length(c{MCindex,nodeIndex,roadmapIndex}) ~= 0)
	sum = sum + c{MCindex,nodeIndex,roadmapIndex};
      else
	numUnusedRoadmap = numUnusedRoadmap + 1;
      end
    end

    if (numUnusedRoadmap ~= numRoadmapsMissing(nodeIndex))
      error('numUnusedRoadmap ~= numRoadmapsMissing(nodeIndex)\n');
    end

    if (p > numRoadmapsMissing(nodeIndex))
      %keyboard;
      avg(MCindex, nodeIndex) = sum/(p - numRoadmapsMissing(nodeIndex));
    else
      avg(MCindex, nodeIndex) = uninitialized;
    end
  end
end
%keyboard;
  

function stdCell = stdOfCellVector(c,  numMCResultsPerNode, numRoadmapsMissing, ...
				   uninitialized)
[m,n,p] = size(c);
%n
%p

if (length(numRoadmapsMissing) ~= n)
  error('length(numRoadmapsMissing) ~= n)\n');
end
%keyboard;
if (m ~= numMCResultsPerNode)
  error ('m should be %d\n', numMCResultsPerNode);
end
%n == numNodeListSize
%p == numRoadmapsPerNumNode
avg = zeros(numMCResultsPerNode, n);

for MCindex = 1:numMCResultsPerNode
  for nodeIndex = 1:n
    v = []; numUnusedRoadmap = 0;
    for roadmapIndex = 1:p
      if (length(c{MCindex,nodeIndex,roadmapIndex}) ~= 0)
	v = [v c{MCindex,nodeIndex,roadmapIndex}];
      else
	numUnusedRoadmap = numUnusedRoadmap + 1;
      end
    end

    if (numUnusedRoadmap ~= numRoadmapsMissing(nodeIndex))
      error('numUnusedRoadmap ~= numRoadmapsMissing(nodeIndex)\n');
    end

    if (p > numRoadmapsMissing(nodeIndex))
      %keyboard;
      stdCell(MCindex, nodeIndex) = std(v);
    else
      stdCell(MCindex, nodeIndex) = uninitialized;
    end
  end
end



%function [avgCorrVector,stdCorrVector,avgL1Vector, stdL1Vector] = 
%[avgCorrVector, stdCorrVector] = aggregateResults(corrVector);
%[avgL1Vector  , stdL1Vector  ] = aggregateResults(L1Vector);


%avgCorrVector = zeros(numNodeListSize, numMCResultsPerNode);
%stdCorrVector = avgCorrVector;
%avgL1Vector   = avgCorrVector;
%stdL1Vector   = avgCorrVector;


%function [avgVector, stdVector] = aggregateResults(inputVector)
%avgVector = mean(inputVector);
%xstdVector = std (inputVector);

