function pf = computePfold(T, f, u, solver, prof)
% pfold
%   Compute the probability of folding
%
%   T:      transition probability matrix for all nodes
%   f:      indices of nodes in the folded state
%   u:      indices of nodes in the unfolded state
%   solver: 1 for direct, 2 for Jacobi, 3 for CG
%   prof:   1 to turn on the profiling, 0 to turn off
%  

if (nargin < 3) 
  error('pfold: too few arguments');
end;
if (nargin < 4)
  solver = 3;
end;  
if (nargin < 5)
  prof = 0;
end

[m,n] = size(T);
pf    = zeros(m,1);
[T, b, pfInd] = getPfoldMat(T,f,u);

% solve the equation  pf = T * pf + b



if (solver==1)				% direct method
  if (prof) profile on; end;
%  keyboard;
  pf(pfInd) = direct(T,b);
%  keyboard;

else					% iterative methods
  pf0 = 0.5 * ones(length(pfInd),1);
% $$$   pf0 = (T \ b) + 0.03*rand(length(pfInd),1);
% $$$   pf0 = T \ b; pf0(1:100:4500) = pf0(1:100:4500)-0.1;
  maxit = 1E6;

  % Don't use the sparse representation for the vectors. It slows down
  % the solver.
  pf0 = full(pf0);
  b   = full(b);

  if (prof) profile on; end; 
  
  if  (solver==2)  			% jacobi 
    pf(pfInd) = jacobi(T, b, [], maxit, pf0);
  elseif (solver==3)			% conjugate gradient
    pf(pfInd) = cgs(T, b, [], maxit, [],[], pf0) ;
  end

end

if (prof) 
  profile report;
  profile off;
end;
pf(f) = 1;
pf(u) = 0;
pf = reshape(pf,length(pf),1);
