function computePfoldIterate (numRoadmapsPerNumNodes,...
			      numNodeList, ...
			      numPfolds2Write,...
			      startRoadmapIndex, startNodeIndex)


%computes pfolds for the roadmaps, and writes them (pfold for the
%nodes startRoadmapIndex .. numPfolds2Write+startRoadmapIndex - 1 (in 1-based
%format, i.e. startNodeIndex >= 1))
%down to "pfold%d_%d.txt", numNodeList(i), j; 
%j = startRoadmapIndex:(startRoadmapIndex+numRoadmapsPerNumNodes-1).

% it is important to set numPfolds2Write properly, so that enough
% number of data points (pfolds) are written to the file, that can
% be compared with MC pfold results.

%e.g., if nodes 2..40 are the same in all roadmaps, numPfolds2Write
%should be set to 41 so that nodes 0..40 pfold results are written.

% (important related note: it is necessary to remove from the MC
% pfold results all pfold results that correspond to nodes that are
% not common).



%the transition probabilities are stored in "transitionProbs%d_%d.txt, numNodeList(i), startRoadmapIndex + j".
%where j = 0:(numRoadmapsPerNumNodes - 1).


if (nargin < 3)
  error('usage: computePfoldIterate (<numRoadmapsPerNumNodes> <numNodeList> <numPfolds2Write> {startRoadmapIndex} {startNodeIndex})');
end

if (nargin < 5)
  startNodeIndex    = 1;
end
if (nargin < 4)
  startRoadmapIndex = 0;
end

if (startNodeIndex == 0)
  fprintf(1, 'start node index should be > 0\n');
  keyboard;
end

%addpath('/u/apaydin/research/mFiles/general');

for i = 1:length(numNodeList)
  for j = startRoadmapIndex:(startRoadmapIndex+numRoadmapsPerNumNodes-1)
    transitionProbsFilename             = sprintf('transitionProbs%d_%d.txt', ...
						  numNodeList(i), j) ;
    
    fprintf        (1, 'reading %s\n', transitionProbsFilename);

    % read folded and unfolded macrostates;
    
    foldedFilename       = sprintf('close2Folded%d_%d.txt',...
				   numNodeList(i), j);
    
    unfoldedFilename     = sprintf('close2Unfolded%d_%d.txt',...
				   numNodeList(i), j);
    
    fprintf(1, 'reading %s and %s\n\n', foldedFilename, unfoldedFilename);
    
      
    foldingP = computePfoldWrapper(transitionProbsFilename, ...
				   foldedFilename, ...
                                   unfoldedFilename, ...
				   startNodeIndex, ...
				   startNodeIndex + numPfolds2Write - 1);
    
    outputFilename = sprintf('pfold%d_%d.txt', numNodeList(i), j);

    fid = fopen  (outputFilename, 'w');
    fprintf      (fid, '\%numnodes = %d roadmap index = %d\n', numNodeList(i), j);
    fprintf      (fid, '\%folding p computed with roadmap, and 1E6 iters.\n');
    if (length(foldingP) > 0)
      fprintf      (fid, '%f\n', foldingP(startNodeIndex:startNodeIndex ...
					  + numPfolds2Write - 1)');
    end
    fclose       (fid);
    fprintf      (1, 'printed results to %s\n', outputFilename);
  end
end


