function retval = findBiggestCC(transitionProbs)

% returns the list of nodes (first node has index 1) in the biggest cc of the roadmap.
% the algorithm is as follows:
% for each node, keeps track of the tree index it is in.
% when two nodes get connected, updates the index of the tree
% such that both nodes as well as the nodes that have the index
% of either tree have the same index, the index of one of the nodes
% trees. 
tmp1               = max(transitionProbs(:,1)) + 1;
tmp2               = max(transitionProbs(:,2)) + 1;

numNodes           = max(tmp1, tmp2);

treeNames = 1:numNodes;
treeNameAssigned = ones(1, numNodes);

%treeNameAssigned(i) becomes 0 when i is a tree name which is no longer used.

[numLinks, ignore] = size(transitionProbs);

for i = 1:numLinks

  node1 = transitionProbs(i,1) + 1;  %converting to 1 based numbers
  node2 = transitionProbs(i,2) + 1;


  if (node1 ~= node2)  
     if (treeNames(node1) ~= treeNames(node2))
       newTreeName     = min(treeNames(node1), treeNames(node2));
       replaceTreeName = max(treeNames(node1), treeNames(node2));
       
       indices            = find(treeNames == replaceTreeName);
       treeNames(indices) = newTreeName;

       treeNameAssigned(replaceTreeName) = 0;
     end
  end
end

       
nonEmptyTrees = find(treeNameAssigned == 1);
fprintf(1, 'num non empty trees = %d\n', length(nonEmptyTrees));
fprintf(1, 'num elements in trees are : \n');

maxNumNodes = 0;
for i = 1:length(nonEmptyTrees)
     numNodesInTree = length(find(treeNames == nonEmptyTrees(i)));
     fprintf(1, 'tree # %d has %d nodes\n', i, numNodesInTree);
     if (numNodesInTree > maxNumNodes)
       maxNumNodes     = numNodesInTree;
       biggestTreeName = nonEmptyTrees(i);
     end
end

retval = find(treeNames == biggestTreeName);

