function [A, b, pfInd] = getPfoldMat(T, f, u)
% getPfold
%   Set up the matrices for pfold computation
%
%   T:     transition probability matrix for all nodes
%   f:     indices of nodes in the folded state
%   u:     indices of nodes in the unfolded state
%   A:     transition probability matrix for the unknowns
%   b:     sum of probabililities for nodes in the folded state
%   pfInd: indices of nodes for the unknowns
%

numNodes = size(T,1);

if ~(all((f >= 1) & (f <= numNodes)) & all((u >= 1) & (u <= numNodes)))
  error('getPfoldMat: invalid node indices in f or u');
elseif ~(all( abs(sum(T,2)-1) < 1e-5 ))
  error('getPfoldMat: invalid transition matrix T');
end

% find the indices of the unknowns
allNodes = 1:numNodes;
goal     = [f, u];
allNodes(goal) = NaN;
pfInd    = allNodes(finite(allNodes));	% indices of unknowns


%we need to prepare A and b, where b = sigma(Pij)/(1 - Pii), where
%j in folded set;
%and A = I - T, where Tij = pij / (1 - pii), j not in folded or
%unfolded set, and Tii = 0.
%pij = Tij if i and j are in pfInd.


% sum up the probabilities for nodes in the folded state
A = T(pfInd, :);			% keep only rows for the unkowns
b = sum(A(:, f),2);
A = A(:, pfInd);			% keep only columns for the unknowns


numUnknowns = length(pfInd);


diagA   = diag(A);
%zeroing the diagonal entries of A
%A       = (1 - eye(numUnknowns)) .* A; this operation doesn't fly
%when numUnknowns >= 10000
for i = 1:numUnknowns %aka size(A)
  A(i,i) = 0;
end


%multiplying each row of A by 1/(1-pii).

denum = zeros(numUnknowns, 1); %holds denum (1-pii)

for i = 1:numUnknowns
  if (diagA(i) == 1)
    fprintf(1, 'diagA(i) = 1');
    keyboard;
  end
  denum(i) = (1-diagA(i));
end

for i = 1:numUnknowns
  A(i, :) = A(i, :) / denum(i);
end

A = speye(numUnknowns) - A; %just eye(numUnknowns) doesn't work when # numUnknowns >= 10000.

b = b ./ denum;


