function [x] = jacobi(A, b, tol, maxit, x0)
% jacobi
%   Jacobi iteration for solving linear systems

if (isempty(tol))
  tol = 1e-6;
end
if (isempty(maxit))
  maxit = 20;
end

n  = size(A,1);
T  = speye(n) - A;
nb = norm(b);

% iterate until convergence
iterBest = 1;
relResBest = norm(A*x0-b) / nb;

solved = 0;				% solution found
for i = 1:maxit
  x = T * x0 + b;
  relRes = norm(A*x-b) / nb;		% relative residual error of x
  if ( relRes < relResBest )
    relResBest = relRes;
    iterBest = i;
  end
  
  if  (relRes < tol)
    solved = 1;
    break;
  end
  
  x0 = x;
end

if (solved == 1)
  fprintf('\njacobi: converged at iteration %d with residual %g\n', i, ...
	  relRes);
else     
  fprintf('\njacobi: failed to converge after maximum %d iterations\n',maxit);
  fprintf('Iteration %d has relative residual %g\n', iterBest, relResBest);
end
