function plotPfoldComparison(numNodeList, ...
			     numRoadmapsPerNumNodes, ...
			     MCpfoldResultsFilename, ...
			     startIndex, relDirName,...
			     numMCResultPerNode)

%plots the {correlation && L1 distance} between MC and SRS pfold values
 
%PARAMETERS


%assumes that SRS pfolds are stored in the current dir., under the
%names relDir/pfold%d_%d.txt, numNodeList(:). startIndex:numRoadmapsPerNumNodes-1
%numMCResultPerNode is 1,2,3 or 10 currently. 


if (nargin < 1)
  error('plotPfold : too few arguments');
end
if (nargin < 2)
  numRoadmapsPerNumNodes = 1;
end
if (nargin < 3)
  MCpfoldResultsFilename = './MCPfoldSummary.txt'; 
end
if (nargin < 4)
  startIndex             = 0;
end
if (nargin < 5)
  relDirName             = '.'; %where the pfold results are stored
end
if (nargin < 6)
  numMCResultPerNode     = 3;
end


%if ((plotType ~= 1) & (plotType ~= 0))
%  error('error! plotType should be either 0(L1 distance) or 1(correlation)\n');
%end

if (numMCResultPerNode < 1) 
  error('error! numMCResultPerNode should be >= 1\n');
end


%comparisonIndices     = [1:length(numNodeList)];



UNINITIALIZED    = -2208;

MCfile = load(MCpfoldResultsFilename);
numMCIterationsPerLine = MCfile(1:numMCResultPerNode, 4);
[numRows, dummy]       = size(MCfile);
numComparisonNodes     = numRows/numMCResultPerNode;


%computing the thing to be plotted
[avgCorrVector, stdCorrVector, avgL1Vector, stdL1Vector] = ...
    compare_SRS_MC_Pfold   (numRoadmapsPerNumNodes, numNodeList, relDirName, ...
			    MCpfoldResultsFilename, startIndex, ...
			    numMCResultPerNode,...
			    UNINITIALIZED);
comparisonIndices          = find(avgCorrVector(1, 1:length(numNodeList)) ~= UNINITIALIZED);
%keyboard;
%now it is time to do the plotting
plotResults(avgCorrVector, stdCorrVector,numNodeList, comparisonIndices, numMCResultPerNode, ...
	    numRoadmapsPerNumNodes, 1, numMCIterationsPerLine, numComparisonNodes);
plotResults(avgL1Vector, stdL1Vector, numNodeList, comparisonIndices, ...
	    numMCResultPerNode, numRoadmapsPerNumNodes, 0, numMCIterationsPerLine,numComparisonNodes);


%%%
function plotResults(avgVector, stdVector, numNodeList, comparisonIndices, ...
		     numMCResultPerNode, numRoadmapsPerNumNodes, plotType, numMCIterationsPerLine,numComparisonNodes)

%plotType == 0 for L1 distance, 1 for correlation
if (plotType ~= 0) & (plotType ~= 1) 
  error('plotType should be 0 or 1');
end

figure;
switch(numMCResultPerNode)
 case 1,
  H1 = plot(numNodeList(comparisonIndices), avgVector(1, ...
						  comparisonIndices), 'r^-', 'LineWidth', 2);
  labelLegends (numMCIterationsPerLine, H1);
 case 2,
  H1 = plot(numNodeList(comparisonIndices), avgVector(1, ...
						  comparisonIndices), 'r^-', 'LineWidth', 2);
  hold
  H2 = plot(numNodeList(comparisonIndices), avgVector(2, ...
						  comparisonIndices), 'b-+','LineWidth', 2);
  labelLegends (numMCIterationsPerLine, H1, H2);
 case 3,
  H1 = plot(numNodeList(comparisonIndices), avgVector(1,comparisonIndices), 'r^-', 'LineWidth', 2);
  hold
  
  H2 = plot(numNodeList(comparisonIndices), avgVector(2, ...
						  comparisonIndices), ...
	    'b-+','LineWidth', 2);
  H3 = plot(numNodeList(comparisonIndices), avgVector(3, ...
						  comparisonIndices), 'g-o','LineWidth', 2);
  labelLegends (numMCIterationsPerLine, H1, H2, H3);
 case 10,
  H1 = plot(numNodeList(comparisonIndices), avgVector(1, ...
						  comparisonIndices), 'r^-', 'LineWidth', 2);
  hold

  H2 = plot(numNodeList(comparisonIndices), avgVector(5, ...
						  comparisonIndices), ...
	    'b-+', 'LineWidth', 2);

  H3 = plot(numNodeList(comparisonIndices), avgVector(10, ...
						  comparisonIndices), 'g-o','LineWidth', 2);
	    
  labelLegends (numMCIterationsPerLine, H1, H2, H3);
 otherwise,
  error('NumMCResultPerNode should be between 1 and 3 or equal to 10\n');
end

%now making the plot more understandable


labelPlot(plotType, numRoadmapsPerNumNodes,numComparisonNodes);

grid

%now plotting standard deviation
figure

%keyboard
Hstd = plot(numNodeList(comparisonIndices), avgVector(numMCResultPerNode, comparisonIndices), ...
	    'r-o', 'LineWidth', 4);
legendString = sprintf('%d MC runs/node', numMCIterationsPerLine(length(numMCIterationsPerLine)));
legend([Hstd], legendString, 0)
hold
%keyboard
errorbar(numNodeList(comparisonIndices), avgVector(numMCResultPerNode, comparisonIndices), ...
	 stdVector(numMCResultPerNode, comparisonIndices), 'b');
h = gca;
set(h, 'LineWidth', 2, 'Fontsize', 15);
%keyboard
labelPlot(plotType, numRoadmapsPerNumNodes,numComparisonNodes)
grid
%keyboard

function labelPlot(plotType, numRoadmapsPerNumNodes,numComparisonNodes)
xlabel(' number of nodes')
if (plotType == 0)
  ylabel('l1 distance') 
else
  ylabel('correlation') 
end

titleString = sprintf('%d roadmap each, protein x, connection ? MC:step size, %d data pts', numRoadmapsPerNumNodes, ...
		    numComparisonNodes);
title(titleString);

function labelLegends (numMCIterationsPerLine, H1, H2, H3 )

h = gca;
set(h, 'LineWidth', 2, 'Fontsize', 15);
%keyboard;
strings      = cell(length(numMCIterationsPerLine), 1);
for i = 1:length(numMCIterationsPerLine)
  strings{i} = sprintf('%d MC runs/node', numMCIterationsPerLine(i));
end

switch(length(numMCIterationsPerLine))
  case 1,
   legend([H1], strings{1},0)
 case 2,
  legend([H1 H2], strings{1}, strings{2}, 0)
 case 3,
  legend([H1 H2 H3], strings{1}, strings{2}, strings{3}, 0)
 case 10,
  legend([H1 H2 H3], strings{1}, strings{5} , strings{10}, 0)
 otherwise,
  error('NumMCResultPerNode should be between 1 and 3 or equal to 10\n');
end
