#include <iostream>
using std::cerr;
using std::endl;
#include "protein.H"
#include "ANN/ANN.h"


ANNpoint	  query_pt;        		// query point
ANNpointArray	  data_pts;	        	// data points
ANNidxArray	  nn_idx;			// near neighbor indices
ANNdistArray	  dists;			// near neighbor distances
ANNkd_tree	  *the_tree;      		// search structure
int               ANN_k = 0;                        // number of nearest neighbors, also MAXLINKS  
unsigned int      ANN_dim = 0;              	// dimension
const  double	  ANN_eps		= 0.0;	// error bound
bool              initializationDone    = false;

///@name ANN.C
//@{
///initialize data structures to do the brute-force search of all nearest neighbors.
void initializekNN(int numNodes, Protein ** allNodes)
{

  ANN_dim  = 2;
  ANN_k    = numNodes - 1;

  assert (ANN_k < numNodes);

  query_pt = annAllocPt(ANN_dim);			// allocate query point
  data_pts = annAllocPts(numNodes, ANN_dim);		// allocate data points
  nn_idx   = new ANNidx [ANN_k];			// allocate near neigh indices
  dists    = new ANNdist[ANN_k];			// allocate near neighbor dists
  
  for (int i = 0; i < numNodes; i++) 
    {
      data_pts[i][0] = allNodes[i]->getX();
      data_pts[i][1] = allNodes[i]->getY();
    
    }
  
  the_tree = new ANNkd_tree(			// build search structure
			    data_pts,		// the data points
			    numNodes,		// number of points
			    ANN_dim);		// dimension of space

  initializationDone = true;
  cerr << "Inialization of the kd-tree is completed... " << endl;
}

/** 
   finds the nearest nodes to a given node, according to C-space
   parameters. 
*/

void findNearestNeighbors(int nodeIndex, Protein ** allNodes) 
{
  assert (initializationDone);
  query_pt [0] = allNodes[nodeIndex]->getX();
  query_pt [1] = allNodes[nodeIndex]->getY();
	

  the_tree->annkSearch(                           // search
		       query_pt,		  // query point
		       ANN_k,
		       nn_idx,		          // nearest neighbors (returned)
		       dists,		          // distance (returned)
		       ANN_eps);	       	  // error bound

  
}
//@}
