#include <vector>
#include <iostream>
using std::cout;
using std::endl;

#include "ioRoutines.H"
#include "matrix.H"
#include "protein.H"
#include "analysis.H"
#include "tree.H"

/**@name analysis.C */
//@{



///computes pfold, writes the result to a file.
void computeAndPrintPfold (vector<Real> & pfolds, int numPfoldsRequested, 
			   int maxNumNeighbors, 
			   int runIndex,
			   float foldedMacrostateRMSDRadius, 
			   float unfoldedMacrostateRMSDRadius, 
			   int numberOfNodes,
			   Protein**  allNodes,
			   int foldedStateIndex, int unfoldedStateIndex,
			   const ccInfoType * ccInfo)
{ 

  vector<int> foldedIndices, unfoldedIndices;
  //cerr << "analysis.C: (using " << ((sizeof(Real)==sizeof(float))? "float": "double") << ")" << endl;
  
  if (numPfoldsRequested > numberOfNodes)
    numPfoldsRequested = numberOfNodes;

  foldedIndices   = computeCloseNodes(foldedStateIndex, allNodes, 
				      numberOfNodes,
				      foldedMacrostateRMSDRadius);

  unfoldedIndices = computeCloseNodes(unfoldedStateIndex, 
				      allNodes, 
				      numberOfNodes,
				      unfoldedMacrostateRMSDRadius);
  
  computePfoldWrapper( pfolds, numberOfNodes, allNodes, 
		       numPfoldsRequested, 
		       foldedIndices,
		       unfoldedIndices,
		       maxNumNeighbors,
		       ccInfo);
  
  if (pfolds.size() != (unsigned int) numPfoldsRequested)
    cout << "pfold size = " << pfolds.size() << " numPfoldsRequested = " << numPfoldsRequested << endl;
  //assert (pfolds.size() == (unsigned int) numPfoldsRequested);
  printPfolds    (pfolds, numberOfNodes,  runIndex);
}

///
vector<int> computeCloseNodes(int centerNodeIndex, Protein ** allNodes, 
			      int numNodes,
			      float macrostateRMSDRadius)
{
  vector<int> retval(0);

  assert ((centerNodeIndex >= 0) && (centerNodeIndex < numNodes));
  
  for (int i = 0; i < numNodes; i++)
    {
      if (allNodes[centerNodeIndex]->computeDistance(allNodes[i]) < macrostateRMSDRadius)
	{
	  retval.push_back(i);
	}
    }
  return retval;
}


/** returns the index of the node w/ min Energy */
int findMinEnergyNode(Protein ** allNodes, int numberOfNodes)
{
  float minE = 10000.0;
  int retval;
  for (int i = 0; i < numberOfNodes; i++)
    {
      float en = allNodes[i]->getEnergy();
      if (en < minE)
	{
	  minE = en;
	  retval = i;
	}
    }
  return retval;
}

///sets every nodes nodeColor field to given thecolor. useful as a flag.
void initNodeColor(const int numNodes, int thecolor, Protein ** allNodes) {

  int i;

  for (i = 0; i < numNodes; i++) {
    allNodes[i]->myNode->nodeColor = (char)(thecolor);
    allNodes[i]->myNode->nodeTime = -1;
  }

}
//@}
