#include <fstream>
using std::ofstream;
#include <vector>
#include <string>
using std::string;
#include <iostream>
using std::cout;
using std::endl;
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <sys/times.h>
#include "protein.H"
#include "roadmapConstruction.H"
#include "matrix.H"
#include "tree.H"






/**@name ioRoutines.C*/
//@{

//------------------read routines--------------------------------


/**
  reads node coords from filename.
  the format is : nodeIndex x_coord y_coord pfold.
  discards node index and pfold.
*/
void readNodeCoords (const char * filename, 
		     vector<float> & xcoords,
		     vector<float> & ycoords)
{
  FILE * fin = fopen(filename, "r");
  if (fin == NULL)
    {
      cout << "could not read " << filename << endl;
      assert (fin != NULL);
    }
  else
    {
      cout << "reading " << filename << endl;
    }

  xcoords.resize(0); ycoords.resize(0);
  
  int dummy1; 
  float x, y, dummy2; 


  while (!feof(fin))
    {
      fscanf(fin, "%d %f %f %f\n", &dummy1, &x, &y, &dummy2);
      xcoords.push_back(x); ycoords.push_back(y);
      //cout << x << " " << y << endl;
    }
  cout << "read " << xcoords.size() << " coords." << endl;
  fclose (fin);
}



///reads pfold100MC.txt and returns nativeNodeIndex, which is 101.
int readPresampledConformations(Protein ** allNodes, 
				int *numNodes)
{
  vector<float> xcoords, ycoords; 
  // coords of nodes, for which pfold is 
  // already computed.

  readNodeCoords     ("pfold100MC.txt", xcoords, ycoords);
  
  cout << "numnodes before creating given configs is : " << *numNodes << endl;
  sampleGivenCoords  (xcoords, ycoords, numNodes, allNodes);
  cout << "numnodes after creating given configs is : " << *numNodes << endl;

  int nativeNodeIndex = *numNodes - 1;
  
  //cout << "native node:" << allNodes[nativeNodeIndex]->getX() << " " << allNodes[nativeNodeIndex]->getY() << endl;
  return nativeNodeIndex;
}



//------------------write routines-------------------------------


/**
   prints dist. from the origin, energy to the filename.
   prints x,y coords to MC_Locs.txt.
   the latter file can be used to show the trajectory on the landscape 
*/
void printPath(const vector<int>& path, Protein ** allNodes,
	       const string& filename)
{
  unsigned int i;
  ofstream fout(filename.c_str());
  ofstream fout2("MC_Locs.txt");


  cout << "check MC_Locs.txt and " << filename << "." << endl; 

  for (i = 0; i < path.size(); i++) 
    {
      float x   = allNodes[path[i]]->getX();
      float y   = allNodes[path[i]]->getY();
      float d2O = sqrt(x * x + y * y); // distance 2 origin
      // fout << allNodes[path[i]]->computeDistance(allNodes[0]) << " " << 
      fout << d2O << " " << allNodes[path[i]]->getEnergy() << endl;
      //fout << allNodes[path[i]]->getX() << " " << allNodes[path[i]]->getY() << endl;
      fout2 << x << " " << y << endl;
    }
  
  fout.close();
  fout2.close();

  

  cout << "check " << filename << endl;
}


///normally this function should be in analysis.C but for some reason it does not link there.
void printPfolds    (const vector<Real>  & pfolds, int numNodes, int runIndex)
{
  char pfoldResultFilename[1000];

  sprintf(pfoldResultFilename, "pfold%d_%d.txt", numNodes, runIndex);
  ofstream fout(pfoldResultFilename);
  
  cout << "check " << pfoldResultFilename << endl;
  
  for (unsigned int i = 0; i < pfolds.size(); i++)
    {
      fout << pfolds[i] << endl;
    }


}


///
void linkNodesStatPrintOuts(int numNodes, 
			    Protein ** allNodes, ccInfoType * ccInfo)
{

  cout << "Nodes Done: " << numNodes << " Num Trees: " << ccInfo->numTrees << endl;
 
  if (ccInfo->globalFirstTree != NULL) 
    ccInfo->globalFirstTree->printTrees(numNodes, allNodes);
  
  
  //if (numTrees == 1)
  //assert (allNodes[0]->myTree->numNodesInTree == numNodes);
}


/**
  prints the indices of nodes that are close to the folded or unfolded nodes
  in a file, in 1-based format.

  the filename is sprintf("close2{Folded|Unfolded}%d_%d.txt", numNodes, 
  runIndex);
*/
void printCloseNodeIndices (int foldedStateIndex, int unfoldedStateIndex,
			    float minDistFolded, float minDistUnfolded, 
			    Protein ** allNodes, int numNodes,
			    int runIndex)
{ 
  
  vector<int>     close2FoldedNodeIndices, close2UnfoldedNodeIndices;
  char filename[100];
  
  close2FoldedNodeIndices  .resize(0);
  close2UnfoldedNodeIndices.resize(0);

  cout << "numNodes = " << numNodes << endl;
  
  for ( int i = 0; i < numNodes; i++)
    {
      if (configsClose(allNodes[i], allNodes[foldedStateIndex], minDistFolded))
	{
	  close2FoldedNodeIndices.push_back(i+1);
	}

      if (configsClose(allNodes[unfoldedStateIndex],allNodes[i], 
		       minDistUnfolded))
      {
	  close2UnfoldedNodeIndices.push_back(i+1);
	}
    }
  
  sprintf        (filename, "close2Folded%d_%d.txt", 
		  numNodes, 
		  runIndex);
  cout << "check " << filename << endl;
  ofstream  fout2(filename);

  fout2 << "% numnodes = " << numNodes << " runIndex = " << runIndex << endl; 
  fout2 << "% radius of the folded macrostate : " << minDistFolded << endl;
  fout2 << "% node indices (1-based) of nodes close 2 folded state nodes are : " << endl;

  for ( unsigned int i = 0; i < close2FoldedNodeIndices.size(); i++)
    {
      fout2 << close2FoldedNodeIndices[i] << " "; 
    }
  fout2 << endl;
  fout2.close();

  sprintf        (filename, "close2Unfolded%d_%d.txt", 
		  numNodes, 
		  runIndex);

  fout2.open(filename);
  cout << "check " << filename << endl;
  fout2 << "% numnodes = " << numNodes << " runIndex = " << runIndex << endl; 
  fout2 << "% radius of the unfolded macrostate : " << minDistUnfolded << endl;
  fout2 << "% node indices (1-based) of close 2 unfolded state nodes are : " << endl;

  for ( unsigned int i = 0; i < close2UnfoldedNodeIndices.size(); i++)
    {
      fout2 << close2UnfoldedNodeIndices[i] << " " ;
    }
  fout2 << endl;
  fout2.close();
}


///
void writeConnections(Protein ** allNodes, int numNodes, 
		      int runIndex)
{

  string        connFilename = "connections";
  char          connFilenameSuffix[100];
  vector<int>   neighborIndices;
  FILE * fout;

  
  sprintf(connFilenameSuffix, "%d_%d.txt", numNodes, runIndex);
  connFilename.append(connFilenameSuffix);

  fout = fopen(connFilename.c_str(), "w");
  if (fout == NULL)
    {
      fprintf(stderr, "%s could not be opened.\n", connFilename.c_str());
      exit(0);
    }
  else
    cout << "check " << connFilename << endl;


  

  for (int i = 0; i < numNodes; i++)
    {
      findNeighbors (neighborIndices,
		     i, 
		     allNodes);
      
      for (unsigned int j = 0; j < neighborIndices.size() ; j++)
	{
	  assert (neighborIndices[j] != i);
	  fprintf(fout, "%d %d \n", i, neighborIndices[j]);
	}
    }
  fclose(fout);
}

///writes the edge transition probabilities to a file titled filename.
void writeTransitionProbabilities(int numNodes, Protein ** allNodes, 
				  const char * filename)
{
  //char filename[] = "transitionProbs.txt";
  FILE * fout     =  fopen (filename,"w");
  vector<float> tranProbs;
  vector<int>   neighborIndices;
  
  cout << "check " << filename << endl;
  for (int i = 0; i < numNodes; i++)
    {
      findNeighborsAndTransitionProbabilities (neighborIndices,
					       tranProbs,
					       i, 
					       allNodes);
      assert (neighborIndices.size() == tranProbs.size());

      for (unsigned int j = 0; j < neighborIndices.size() ; j++)
	{
	  assert (tranProbs[j] >= 0);
	  assert (neighborIndices[j] != i);
	  fprintf(fout, "%d %d %e \n", i, neighborIndices[j], tranProbs[j]);
	}
    }
  fclose(fout);
}

///to stdout
void writeNodes(Protein ** allNodes, int numNodes)
{
  cout << "the nodes are at: " << endl;
  for (int i = 0; i < numNodes; i++)
    {
      cout << allNodes[i]->getIndex() << " : ";
      cout << allNodes[i]->getX() << " " << allNodes[i]->getY() << endl;
    }
}

///
void writeNodes(Protein ** allNodes, int numNodes, int runIndex)
{
  string nodeFilename = "nodes";
  char nodeFilenameSuffix[100];
  sprintf(nodeFilenameSuffix, "%d_%d.txt", numNodes, runIndex);
  nodeFilename.append(nodeFilenameSuffix);

  FILE * fout;
  
  fout = fopen(nodeFilename.c_str(), "w");
  if (fout == NULL)
    {
      fprintf(stderr, "%s could not be opened.\n", nodeFilename.c_str());
      exit(0);
    }
  
  for (int i = 0; i < numNodes; i++)
    {
      fprintf(fout, "%f  %f \n", allNodes[i]->getX(),allNodes[i]->getY());
    }
  fclose(fout);
  cout << "check " << nodeFilename.c_str() <<endl;
}



///
void reportTimes (struct tms start, struct tms end)
{
  cout << "this function is unchecked (ioRoutines::reportTimes)" << endl;
  float ticks =   sysconf(_SC_CLK_TCK);//CLK_TCK;
  float utime = (end.tms_utime - start.tms_utime) / ticks;
  float stime = (end.tms_stime - start.tms_stime) / ticks;
  cout << "\t User time\t= " << utime << endl;
  cout << "\t System time\t= " << stime << endl;
  cout << "\t Total time\t= " << utime + stime << "\n\n";
  
}


//-----------------------other helpers------------------------------

///
void handleInput (int argc, char ** argv, int * numRandomNodes, //int * numLM,
		  int * index, int MAXNODES)
{
  
  if ((argc < 2)  || (argc > 3)){
    cout << "Usage: " << argv[0] << " <numRandomNodes> (index?) " << endl;
    exit(0);
  }

  if (argc == 3) 
    {
      (*index) = (int) atoi(argv[2]);
    }
  else
    {
      (*index) = 0;
    }
  cout << "index = " << *index << endl;
	
  (*numRandomNodes) = (int)(atoi(argv[1]));
  
  if ((*numRandomNodes) >= MAXNODES) {
    cout << "Decrease numNodes or increase MAXNODES..." << endl;
    exit(1);
  }
}



//@}
