#include "protein.H"
#include <iostream>
#include "debug.H"
#include "roadmapConstruction.H"
#include "MCSimulation.H"
#include "ioRoutines.H"
using std::cout;
using std::cerr;
using std::endl;
#include <unistd.h>
#include <sys/types.h>
#include "analysis.H"

/** <b>synthetic</b></br>
    <em>created on \today</em>
*/

/**@name predict.C */
//@{
/* constants */
const int    MAXNODES        = 100001;

class Protein;
Protein*      allNodes[MAXNODES];

///
int main(int argc, char ** argv) 
{
  bool totallyRandomNodes = false;

  int numberOfNodes = 0,
    numRandomNodes,
    foldedStateIndex = 0, unfoldedStateIndex = 1,
    //maxIter = (int) 1E6,
    //maxNumMCSimulations = 1000,
    kNN = 4, // trying to connect each node to at least k-NN.
    index,
    numPfoldsRequested = 101;
    
  float macrostateRadius = 1.0;
  vector<Real> pfolds;
  unsigned int maxNumNeighbors,
    seed =   time(0) *getpid(); 

  ccInfoType ccInfo;
  char trProbsFilename[1000];

  
  Protein::loadEnergyFunctionParameters  ("parameters.txt");
  
  cerr << "seed is random  and equal to " << seed << endl;
  srandom(seed);

  handleInput         (argc, argv, &numRandomNodes, &index, 
		       MAXNODES);



  sampleAllNodes      (&numberOfNodes,  numRandomNodes, &foldedStateIndex, 
		       &unfoldedStateIndex, allNodes, MAXNODES, 
		       totallyRandomNodes);

  checkProteins       (allNodes, numberOfNodes);
  cout << "Sampling complete.  NumNodes = " << numberOfNodes << endl;
  
  maxNumNeighbors = linkNodes(numberOfNodes, allNodes, kNN, &ccInfo);

  computeAndPrintPfold (pfolds,  numPfoldsRequested, 
			maxNumNeighbors, 
			index,
			macrostateRadius,
			macrostateRadius,
			numberOfNodes,
			allNodes,
			foldedStateIndex,  unfoldedStateIndex,
			&ccInfo);

  sprintf(trProbsFilename, "transitionProbs%d_%d.txt", numberOfNodes, index);
  writeTransitionProbabilities( numberOfNodes, allNodes, 
				trProbsFilename);
  printCloseNodeIndices       (foldedStateIndex, unfoldedStateIndex,
			       macrostateRadius, macrostateRadius, 
			       allNodes, numberOfNodes, index);
  
  writeNodes(allNodes, numberOfNodes, index);
  /*
    MCpfoldComputation (allNodes[foldedStateIndex], 
    allNodes[unfoldedStateIndex], 
    allNodes[startIndex],   
    macrostateRadius,
    macrostateRadius,
    maxIter,
    maxNumMCSimulations,
    startIndex);
  */
}
//@}
/*@Include:  roadmapConstruction.C ioRoutines.C */
/*@Include: analysis.C matrix.C tree.C protein.C */
/*@Include: MCSimulation.C ANN.C */
