#include <vector>
#include <algorithm>
using std::min;
#include <iostream>
using std::cout;
using std::endl;
#include "protein.H"
#include "tree.H"
#include "ANN/ANN.h"
#include "ioRoutines.H"
#include "nearestNeighbors.H"
#include "roadmapConstruction.H"



/**@name roadmapConstruction.C*/
//@{


//Number of divisions to construct the bins




extern ANNidxArray   nn_idx;
extern ANNdistArray  dists;


//------------------------------------------------------------------
//--------------node creation---------------------------------------

/**
   if (totallyRandom) is false, 
     reads presampled conformations and add in 1 nodes at -50,-50.
   
   creates random samples, 
*/
void sampleAllNodes(int * numberOfNodes, 
		    const int numRandomNodes,
		    int * nativeNodeIndex,
		    int * unfoldedStateIndex,
		    Protein ** allNodes,
		    int MAXNODES,
		    bool totallyRandom)
{
  vector<float> coords(2);

  if (!totallyRandom)
    {
      *nativeNodeIndex = readPresampledConformations(allNodes, numberOfNodes);
    }
  else
    {
      *nativeNodeIndex = *unfoldedStateIndex = -1;
    }
  createRandomNodes                                   (numberOfNodes,  allNodes, numRandomNodes);

  if (!totallyRandom)
    {
      assert ((allNodes[*nativeNodeIndex]->getX() == 0.0) && 
	      (allNodes[*nativeNodeIndex]->getY() == 0));
      
      coords[0] = -50.0; coords[1] = -50.0;
      
      allNodes[*numberOfNodes] = new Protein(*numberOfNodes, coords);
      (*numberOfNodes) ++;
      *unfoldedStateIndex = *numberOfNodes - 1;

      assert ((allNodes[*unfoldedStateIndex]->getX() == -50) && 
	      (allNodes[*unfoldedStateIndex]->getY() == -50));

      cout << "native at " << *nativeNodeIndex << " and unfolded at : " << *unfoldedStateIndex << endl;
    }
  assert ((*numberOfNodes)  < MAXNODES);
}

///
void createRandomNodes (int * numberOfNodes, Protein ** allNodes, 
			int numNodes2Sample)

{
  int   finalNumNodes;
  int   startNumNodes = (* numberOfNodes);
  finalNumNodes       = (* numberOfNodes) + numNodes2Sample;

  while ( (* numberOfNodes) < finalNumNodes)
    {
      allNodes[* numberOfNodes] = new Protein (* numberOfNodes);
      assert(allNodes[* numberOfNodes]->getEnergy() < Protein::maxEnergy);
      (* numberOfNodes)++;
    }

  cout << "only created " << (*numberOfNodes) - startNumNodes << " nodes." << endl;

}

///
void sampleGivenCoord(int *numberOfNodes,
		      Protein ** allNodes,
		      const vector<float> & coords)
{
  allNodes[*numberOfNodes] = new Protein(*numberOfNodes, coords);

  (*numberOfNodes)++;
}

/** creates given nodes */
void sampleGivenCoords(vector<float> & xcoords, 
		       vector<float> & ycoords, 
		       int * numberOfNodes,
		       Protein ** allNodes)
{
  
  vector<float> coords(2);
  assert (xcoords.size() == ycoords.size());

  for (unsigned int i = 0; i < xcoords.size(); i++)
    {
      coords[0] = xcoords[i]; coords[1] = ycoords[i];

      sampleGivenCoord(numberOfNodes, allNodes, coords);
    }
  cout << "finished sampling given coords..." << endl;
}


//---------------------------------------------------------------------------
//---------------------connection--------------------------------------------

///connects node # nodeIndex to k nearest neighbors, unless it is already connected to at least k nearest neighbors.
void kNNLinkNode(int nodeIndex, Protein ** allNodes, int k, int numNodes, 
		 ccInfoType * ccInfo)
		 
{
  findNearestNeighbors(nodeIndex, allNodes);

  for (int i = 0; (i <  numNodes - 1) && 
	 (allNodes[nodeIndex]->getNumLinks() < k);   
       i++) 
    {
      int otherNodeIndex  = nn_idx[i];
      if (!connected(nodeIndex, otherNodeIndex,allNodes))
	{
	  makeLinkBetweenNodes (numNodes, nodeIndex,otherNodeIndex,
				allNodes, ccInfo);
	}
    }
	  
  if (allNodes[nodeIndex]->myTree == NULL)
    addNode2Forest(nodeIndex, allNodes, ccInfo);
}


///links all nodes,  each to kNN other nodes.
unsigned int linkNodes     (int numNodes, 
		    Protein ** allNodes, int k, 
		    ccInfoType * ccInfo)
{
  unsigned int maxNumNeighbors;

  assert (numNodes > 0);
  assert (k > 0);

  initializekNN (numNodes, allNodes);

  for (int i = 0; i < numNodes; i++)
    {
      kNNLinkNode(i, allNodes, k, numNodes, ccInfo);
    }
  
  linkNodesStatPrintOuts( numNodes,  allNodes, ccInfo);
  maxNumNeighbors = computeTransitionProbabilities(allNodes, numNodes);
  return maxNumNeighbors;
}



/**
   connects 2 nodes (if they are not already connected), and updates the tree structure arising from the merge.

   returns the change in the number of trees available in the heap.  For example, if two nodes which are part 
   of two existing trees are connected, their respective trees will be merged and thus the number of trees 
   will be diminished by 1, thus the return value is -1.
*/
int makeLinkBetweenNodes ( int numNodes, const int n1, const int n2, 
			   Protein ** allNodes, ccInfoType * ccInfo)

{

  int retval = 0;

  //cout << "connecting " << n1 << " and " << n2 << endl;

  if (connected(n1, n2, allNodes))
    return 0;
    
  allNodes[n1]->myNode->addNodeLink(allNodes[n2]->myNode);
  allNodes[n2]->myNode->addNodeLink(allNodes[n1]->myNode);
  
  retval = updateTreesOfConnectedPairOfNodes(n1, n2, numNodes, allNodes, ccInfo);

  return retval;
}




/**
  node # count is not connected to the rest of the roadmap,
  creating a tree for itself alone
*/

void addNode2Forest(int count, Protein ** allNodes, ccInfoType * ccInfo)
{
  //	      tempCreated++;
  treeClass  *  tempTreePtr;
  
  tempTreePtr                 = new treeClass(allNodes[count]->myNode,
					      1,
					      ccInfo);
  if (ccInfo->globalLastTree == NULL) 
    {
      ccInfo->globalLastTree  = tempTreePtr;
      ccInfo->globalFirstTree = tempTreePtr;
    }
  else 
    {
      ccInfo->globalLastTree->nextTree = tempTreePtr;
      ccInfo->globalLastTree           = tempTreePtr;
    }
	
  ccInfo->numTrees++;
}	


///assigns tr. prob. to the edge. Note: the assigned tr. prob. right now is different from the one in SRS jcb paper. I should change this.
void findTransitionProbability(int n1, int n2,float E1, float E2, 
			       float *fTP, float *bTP)

{
  float p = ((float)n1)/n2 * exp(-(E2 - E1) * inv_kT);
  
  assert (p != 0.0);
  *fTP = min<float>(1, p);
  *bTP = min<float>(1, 1/p);
}

void  assignTransitionProbability(int i,int j, float fTP, Protein ** allNodes)
{
  bool found;
  ptrList * linkPtr;
  
  linkPtr  = allNodes[i]->myNode->firstLink;
  found    = false;
  
  while ((linkPtr != NULL) && (!found))
    {
      int index = linkPtr->node->myConfig->getIndex();
      
      if (index == j)
	{
	  found = true;
	  assert (linkPtr->pathWeight == 0.0);
	  linkPtr->edgeTransitionProbability = fTP;
	}
      linkPtr = linkPtr ->nextPtr;
    }
  assert (found);
  
}

/// returns maxNumNeighbors of any node.
unsigned int computeTransitionProbabilities(Protein ** allNodes, int numNodes)
{
  vector<int> neighbors;
  float fTP, bTP;
  unsigned int maxNumNeighbors = 0;

  for (int node1 = 0; node1 < numNodes; node1++)
    {
      findNeighbors (neighbors, node1, allNodes);

      if (neighbors.size() > maxNumNeighbors)
	maxNumNeighbors = neighbors.size();
      
      for (unsigned int k = 0; k < neighbors.size(); k++)
	{
	  int node2 = neighbors[k];
	  if (node2 > node1)
	    {
	      findTransitionProbability(allNodes[node1]->getNumLinks(),
					allNodes[node2]->getNumLinks(),
					allNodes[node1]->getEnergy(), 
					allNodes[node2]->getEnergy(), 
					&fTP, &bTP);
					  
	      assignTransitionProbability(node1,node2, fTP, allNodes);
	      assignTransitionProbability(node2,node1, bTP, allNodes);
	    }
	}
    }
  cout << "max. # of neighbors of a node is " << maxNumNeighbors << endl;
  return maxNumNeighbors;
}


//--------------------------------------------------------------------
//----------------------other-----------------------------------------

ccInfoType::ccInfoType()
{
  globalFirstTree = globalLastTree = NULL;
  numTrees        = globalTreeName = 0;
}

//@}
