#ifndef  _TREE_H
#define  _TREE_H

#include <vector>
using std::vector;

#include <set>
using std::set;

#include "roadmapConstruction.H"

class ptrList;
class nodeClass;
class treeClass;
class Protein;
class ccInfoType;

class ptrList {

public:
  ptrList   *nextPtr, *prevPtr;
  nodeClass *node;
  float      pathWeight;
  float      edgeTransitionProbability;
  ptrList();
  
};

class treeClass 
{
public:
  nodeClass *root;
  treeClass *nextTree;
  treeClass *prevTree;
  int        numNodesInTree;
  int        myTreeName;
  // constructors
  treeClass();
  treeClass(nodeClass * rootNode, int numNodes, ccInfoType * ccType);
	    
  void removeTree (treeClass* replacementTree,
		   ccInfoType * ccInfo,
		   Protein   ** allNodes, 
		   const int    numNodes);
  void printTrees (int, Protein **);
  void findElementsInTree(set<int> & elements, int numNodes, 
			  Protein ** allNodes);
};

struct queStruct {

  struct queStruct *nextQEntry;
  float timeStamp;
  nodeClass *fromNode;
  nodeClass *toNode;
};

typedef struct queStruct queType;

class nodeClass {

public:
  ptrList *firstLink;
  ptrList *lastLink;
  Protein *myConfig;
  nodeClass *pathPtr;
  char nodeColor;
  float nodeTime;

  nodeClass();
  void addNodeLink          (nodeClass*, float wt = 0.0);
  void removeAllLinks();
  void replaceTreePtr       (treeClass*, nodeClass*);
  void markAllNodesInTheTree(int * numNodesInTree);
  bool removeConnectionTo   (nodeClass * toNode);
  void printTree();
};


void findNeighbors (vector<int> &neighborIndices, int startNode, 
		    Protein ** allNodes);
unsigned int findNumNeighbors (int i, Protein ** allNodes);
void findNeighborsAndTransitionProbabilities (vector<int>   &neighborIndices,
					      vector<float> &tranProbs,
					      int startNode,
					      Protein ** allNodes);
bool removeLinkBetweenAndUpdateTrees (int n1, int n2, int numNodes,
				      Protein ** allNodes, 
				      ccInfoType * ccInfo);

int updateTreesOfConnectedPairOfNodes (int n1, int n2, int numNodes, 
				       Protein ** allNodes, 
				       ccInfoType * ccInfo);
bool connected (int n1, int n2, Protein ** allNodes);
void  appendNewTreePtr(ccInfoType * ccInfo, treeClass * newTreePtr);
treeClass * findBiggestCC(const ccInfoType * ccInfo);
#endif

