#ifndef _PROTEIN_H
#define _PROTEIN_H

#include <vector>
#include <stdio.h>

const float MY_RAND_MAX =  2147483647.0;
using std::vector;
using std::ostream;

struct neighbourStruct {
    int index;
    float dist;
    int linkedBit;
};

typedef struct neighbourStruct neighbourType;

class treeClass;
class nodeClass;

class Protein 

{

public:
  Protein  (); 
  Protein  (int);
  Protein  (int index, const Protein * original, int * numIterOverall = NULL, 
	    const int maxIter = 100);
  Protein  (int index, const vector<float>& coords);
  ~Protein ();

  //other methods

  void   computeParams      ();
  float  computeEnergy();            
  bool   probabilisticAcceptConformation() const;
  float  computeDistance(const Protein * p) const;
  float  computeDistance(const vector<float>& B) const;
  inline void testInvariant() const throw();
  
  // accessors
  float  getEnergy()   const;         
  float  getX()        const;
  float  getY()        const;
  vector<float>  getParams() const;
  int    getIndex()    const;
  vector<float> getCoords() const;
  nodeClass * getMyNode();
  int    getNumLinks    () const;
  
  //mutators
  void   setXY          (float x, float y);
  void   setNumLinks    (int n);
  void   setEnergy      (float iEnergy);
  void   setIndex       (int index);
  void   setMyNode      (nodeClass * node);

  
  // initialization

  static void   loadEnergyFunctionParameters(const char * filename);
  
  // I/O
  void          reportEnergy(); 
  void          printParameters() const ;
  void          printParameters(FILE * fout) const ;

  //public variables
  treeClass *myTree;             
  nodeClass *myNode;             
  vector<float> params;               

  //static variables
  
  static const float  maxX, maxY, 
    maxEnergy, minEnergy, resolution,deltaX, deltaY;
  static const unsigned int   NUMDOF;
  static int   numEnergyComputations;

 
protected:



private:

  float  x,y;
  float  energy;
  int    numLinks;      // i.e. num neighbors.
  int    index;        /* index to reach this protein in allNodes array.*/
  static vector<float> alphas, taus, xos, yos;

  friend float computeConfigEnergy(const vector<float>& params);
  friend   ostream& operator<< (ostream & output, const Protein& p);

};

//helper function prototypes


//sampling related
void          sampleAroundCoords         (vector<float> &, 
					  const vector<float> &);
vector<float> generateRandomConfig       ();


//other
float computeConfigEnergy               (const vector<float>& params);
bool  checkCollision                    (float energy);
float computeConfigDistance             (const vector<float>& A, 
					 const vector<float>& B);

void midConfig (vector<float> & mid, 
		const vector<float> & start,
		const vector<float> & end);

bool configsClose(const vector<float>&  config1, 
		  const vector<float>&  config2,
		  const float epsilon);

bool configsClose(const Protein * p1, 
		  const Protein * p2,
		  const float epsilon);
void sampleRandomCoords(float * x, float * y);

void coords2Params      (vector<float> & newParams, 
			 const vector<float> & coords);
void params2Coords (vector<float>& coords, 
		    const vector<float> & sampled_params);

inline float energyTerm (float alpha,float tau,  float x_o, float y_o, 
			 float xI, float yI);
#endif
